/*-------------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  hdc1080.c - HDC1080 I2C sensor support functions

  Processor: ATmegaXXX
      Clock: 8 mHz internal

  2018-12-23 written
-------------------------------------------------------------------*/
#include <avr/io.h>
#include <util/delay_basic.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "hdc1080.h"
#include "i2cmaster.h"

#define WANT_DIGITAL_FILTER
#define DIG_SAMPLES		7
#define DIG_ADD			  0        // digital filter states
#define DIG_GET			  1
#define DIG_SORT		  2

#ifdef WANT_DIGITAL_FILTER
static int Compare( const void *pElement1, const void *pElement2 );
static uint16_t digital_filter( uint8_t ucRequest, uint16_t uiSample );
#endif

/*-----------------------------------------------------------------
  hdc1080_write_config(...)

  Input: high-order 8 bits of configuration register
         (low-order 8 bits always zero)

  returns: 0 = OK
           1 = START failed
           2 = write pointer byte fail
           3 = write config 1st byte fail
           4 = write config 2nd byte fail
----------------------------------------------------------------*/
unsigned char hdc1080_write_config( unsigned char ucConfigReg )
{
  unsigned char ucRc;

  // Access the bus and device, output the address
  ucRc = i2c_start( HDC1080_ADDRESS_W );
  if ( ucRc == 1 )             // fail
    return 1;

  // Output the register pointer (device ID)
  ucRc = i2c_write( HDC1080_CONFIG_PTR );
  if ( ucRc == 1 )             // fail
  {
    i2c_stop();
    return 2;
  }

  // write the configuration register, 1st byte
  ucRc = i2c_write( ucConfigReg );
  if ( ucRc == 1 )             // fail
  {
    i2c_stop();
    return 3;
  }

  // write the configuration register, 2nd byte
  ucRc = i2c_write( 0 );
  i2c_stop();

  if ( ucRc == 1 )             // fail
    return 4;

  return 0;
}

/*-----------------------------------------------------------------
  hdc1080_read_humidity(...)

  returns: humidity in percent
           0 = OK
           1 = 1st START failed
           2 = write pointer byte fail
           3 = 2nd START failed
----------------------------------------------------------------*/
unsigned char hdc1080_read_humidity( unsigned *puiHumidity )
{
  unsigned char ucRc;
  unsigned int  uiHumidity;
  unsigned long ulVal1;

  // Access the bus and device, output the address
  ucRc = i2c_start( HDC1080_ADDRESS_W );
  if ( ucRc == 1 )             // fail
    return 1;

  // Output the register pointer (device ID)
  ucRc = i2c_write( HDC1080_HUMIDITY_PTR );
  i2c_stop();
  if ( ucRc == 1 )             // fail
    return 2;

  _delay_loop_2 ( 32768 );         // wait 16ms for conversion

  // Access the bus and device, output the address
  ucRc = i2c_start( HDC1080_ADDRESS_R );
  if ( ucRc == 1 )             // fail
    return 3;

  // read the humidity register, 1st byte
  ucRc = i2c_read( 1 /* read followed by ack */ );
  uiHumidity = (unsigned int) ucRc << 8;

  // read the humidity register, 2nd byte
  ucRc = i2c_read( 0 /* read followed by stop */ );
  uiHumidity = uiHumidity | ucRc;

  i2c_stop();

  // Convert the value received to percent humidity
  // See HDC1080 data sheet for this formula

  ulVal1 = (unsigned long) uiHumidity * 100;
  uiHumidity = (unsigned int) ( ulVal1 / 65536 );     
  *puiHumidity = uiHumidity;

  return 0;
}

/*-----------------------------------------------------------------
  hdc1080_read_temperature(...)

  returns: temperature in 10ths degrees C
           0 = OK
           1 = 1st START failed
           2 = write pointer byte fail
           3 = 2nd START failed
----------------------------------------------------------------*/
unsigned char hdc1080_read_temperature( int *piTemp )
{
  unsigned char ucRc;
  unsigned int  uiTempReg;
  int           iTemp;
  long          lVal1, lVal2;

  // Access the bus and device, output the address
  ucRc = i2c_start( HDC1080_ADDRESS_W );
  if ( ucRc == 1 )             // fail
    return 1;

  // Output the register pointer (device ID)
  ucRc = i2c_write( HDC1080_TEMPERATURE_PTR );
  i2c_stop();
  if ( ucRc == 1 )             // fail
    return 2;

  _delay_loop_2 ( 32768 );         // wait 16ms for conversion

  // Access the bus and device, output the address
  ucRc = i2c_start( HDC1080_ADDRESS_R );
  if ( ucRc == 1 )             // fail
    return 3;

  // read the temperature register, 1st byte
  ucRc = i2c_read( 1 /* read followed by ack */ );
  uiTempReg = (unsigned int) ucRc << 8;

  // read the temperature register, 2nd byte
  ucRc = i2c_read( 0 /* read followed by stop */ );
  uiTempReg |= (unsigned int) ucRc;

  i2c_stop();

#ifdef WANT_DIGITAL_FILTER
  // Filter the sample
  (void) digital_filter( DIG_ADD, uiTempReg );
  (void) digital_filter( DIG_SORT, 0 );
  uiTempReg = (unsigned) digital_filter( DIG_GET, 0 );
#endif

  // Convert the value received to degrees C
  // Make it 100ths of degrees so we can use integer arithmetic.
  // See HDC1080 data sheet for this formula

  lVal1 = (long) (unsigned long) uiTempReg * 165;
  lVal2 = lVal1 / 6550;      // should be 65536 but we want 10ths
  iTemp = (int) ( lVal2 - 400 );

  *piTemp = iTemp;
  return 0;
}

#ifdef WANT_DIGITAL_FILTER
/*-----------------------------------------------------------------
  digital_filter(...)

  ucRequest = DIG_ADD

	Set ucRequest to 0 and pass the sample in uiSample.

  ucRequest = DIG_SORT

	Pass 2 in ucRequest and the array is sorted.

  ucRequest = DIG_GET

	Pass 1 in ucRequest and the median sample is returned.
-----------------------------------------------------------------*/
static uint16_t digital_filter( uint8_t ucRequest, uint16_t uiSample )
{
  static volatile uint16_t uiSamples[DIG_SAMPLES];
  static volatile uint8_t  ucFirst=1;
  uint8_t	ii;
  uint16_t	uiRet;

  if ( ucRequest == DIG_ADD )
  {
	  if ( ucFirst )
	  {
	    // Initialize the table with first value
	    for ( ii=0; ii<DIG_SAMPLES; ++ii )
	      uiSamples[ii] = uiSample;
	    ucFirst = 0;
	    return 0;
	  }

	  // A median filtering method is used, in which the new sample
	  // is placed in the first and last positions of the array, and
	  // the array is sorted in ascending order.

    uiSamples[0] = uiSample;
    uiSamples[DIG_SAMPLES-1] = uiSample;
	  return 0;
  }

  if ( ucRequest == DIG_GET )
  {
  	// Return the median element
	  uiRet = uiSamples[DIG_SAMPLES/2];
	  return uiRet;
  }

  if ( ucRequest == DIG_SORT )
	  qsort( (void *) &uiSamples, DIG_SAMPLES, sizeof( uint16_t ), Compare );

  return 0;
}

/*--------------------------------------------------------------------------
  Compare(...)
--------------------------------------------------------------------------*/
static int Compare( const void *pElement1, const void *pElement2 )
{
  uint16_t *pui1, *pui2;

  pui1 = (uint16_t *) pElement1;
  pui2 = (uint16_t *) pElement2;

  if ( *pui1 > *pui2 ) return 1;
  if ( *pui1 < *pui2 ) return -1;
  return 0;
}
#endif
