/*-------------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  temperature-humidity meter by Jack Botner

  lcd.c - lcd routines

  These routines support the LCD-S301C31TR display. This display
  has 3 digits just like an LED display, and access to each
  segment is wired directly to the microprocessor. So we need to
  translate each number to the corresponding segments and turn
  them on.

  Sorry, ports are all hard-coded.
-------------------------------------------------------------------*/
#include <avr/io.h>
#include "common.h"

/*---------------------------------------------------------------------
  LcdOutputDecimal(...)

  Turns decimal point DP1 or DP2 on or off.

  Input: 0 (none) or 1 (DP1) or 2 (DP2) 
---------------------------------------------------------------------*/
void LcdOutputDecimal( uint8_t ucDp )
{
  if ( ucDp == 1 )
  {
	  clear_bit( PORTD, PD7 );  // DP2
    set_bit( PORTC, PC7 );    // DP1
  }
  else
    if ( ucDp == 2 )
	  {
	    clear_bit( PORTC, PC7 );
	    set_bit( PORTD, PD7 );
	  }
	  else
  	{
	    clear_bit( PORTC, PC7 );
	    clear_bit( PORTD, PD7 );
	  }
}

/*---------------------------------------------------------------------
  LcdClear(...)

  Clears the LCD.
---------------------------------------------------------------------*/
void LcdClear()
{
  clear_bit( PORTA, PA0 );	// A3
  clear_bit( PORTA, PA1 );	// B3
  clear_bit( PORTA, PA2 );	// C3
  clear_bit( PORTA, PA3 );	// D3
  clear_bit( PORTA, PA4 );	// E3
  clear_bit( PORTA, PA5 );	// F3
  clear_bit( PORTA, PA6 );	// G3
  clear_bit( PORTB, PB0 );	// A1
  clear_bit( PORTB, PB1 );	// B1
  clear_bit( PORTC, PC2 );	// C1
  clear_bit( PORTC, PC3 );	// D1
  clear_bit( PORTC, PC4 );	// E1
  clear_bit( PORTC, PC5 );	// F1
  clear_bit( PORTC, PC6 );	// G1
  clear_bit( PORTC, PC7 );  // DP1
  PORTD = 0;                // A2..G2, DP2
}

/*---------------------------------------------------------------------
  LcdOutputNumber(...)

  Outputs a number to the lcd display. Setting the port turns the
  corresponding segment on.

  Input: 1. A single numeric digit, range 0..9 (ascii or binary, only the
            low-order nibble is processed.) Also the value of x'0f'
		        which means blank the display.

	    	 2. Which digit to display, 1 (high order) .. 3 (low order)
---------------------------------------------------------------------*/

// Bit positions in the table of segments
#define SEG_A   0
#define SEG_B   1
#define SEG_C   2
#define SEG_D   3
#define SEG_E   4
#define SEG_F   5
#define SEG_G   6

void LcdOutputNumber( uint8_t ucNumber, uint8_t ucDigit )
{
  // Segment A = bit 0, and so on, for each digit 0..9
  static const uint8_t ucDigTab[10] =
  {
  	0b00111111, 0b00000110, 0b01011011, 0b01001111, 0b01100110,
  	0b01101101, 0b01111101, 0b00000111, 0b01111111, 0b01100111
  };

  uint8_t	ucNibble, ucSegments;

  if ( ucNumber == '-' )			// to display minus sign
    ucSegments = 0b01000000;
  else
  {
	  ucNibble = (unsigned char) ucNumber & 0x0f;
	  ucSegments = ( ucNibble > 9 ) ? 0 : ucDigTab[ucNibble];
  }

  if ( ucDigit == 1 )
  {
    if ( test_bit( ucSegments, SEG_A ) )
      set_bit( PORTB, PB0 );
    else
      clear_bit( PORTB, PB0 );

    if ( test_bit( ucSegments, SEG_B ) )
      set_bit( PORTB, PB1 );
    else
      clear_bit( PORTB, PB1 );

    if ( test_bit( ucSegments, SEG_C ) )
      set_bit( PORTC, PC2 );
    else
      clear_bit( PORTC, PC2 );

    if ( test_bit( ucSegments, SEG_D ) )
      set_bit( PORTC, PC3 );
    else
      clear_bit( PORTC, PC3 );

    if ( test_bit( ucSegments, SEG_E ) )
      set_bit( PORTC, PC4 );
    else
      clear_bit( PORTC, PC4 );

    if ( test_bit( ucSegments, SEG_F ) )
      set_bit( PORTC, PC5 );
    else
      clear_bit( PORTC, PC5 );

    if ( test_bit( ucSegments, SEG_G ) )
      set_bit( PORTC, PC6 );
    else
      clear_bit( PORTC, PC6 );

	  return;
  }

  if ( ucDigit == 2 )
  {
    if ( test_bit( ucSegments, SEG_A ) )
      set_bit( PORTD, PD0 );
    else
      clear_bit( PORTD, PD0 );

    if ( test_bit( ucSegments, SEG_B ) )
      set_bit( PORTD, PD1 );
    else
      clear_bit( PORTD, PD1 );

    if ( test_bit( ucSegments, SEG_C ) )
      set_bit( PORTD, PD2 );
    else
      clear_bit( PORTD, PD2 );

    if ( test_bit( ucSegments, SEG_D ) )
      set_bit( PORTD, PD3 );
    else
      clear_bit( PORTD, PD3 );

    if ( test_bit( ucSegments, SEG_E ) )
      set_bit( PORTD, PD4 );
    else
      clear_bit( PORTD, PD4 );

    if ( test_bit( ucSegments, SEG_F ) )
      set_bit( PORTD, PD5 );
    else
      clear_bit( PORTD, PD5 );

    if ( test_bit( ucSegments, SEG_G ) )
      set_bit( PORTD, PD6 );
    else
      clear_bit( PORTD, PD6 );

	  return;
  }

  if ( ucDigit == 3 )
  {
    if ( test_bit( ucSegments, SEG_A ) )
      set_bit( PORTA, PA0 );
    else
      clear_bit( PORTA, PA0 );

    if ( test_bit( ucSegments, SEG_B ) )
      set_bit( PORTA, PA1 );
    else
      clear_bit( PORTA, PA1 );

    if ( test_bit( ucSegments, SEG_C ) )
      set_bit( PORTA, PA2 );
    else
      clear_bit( PORTA, PA2 );

    if ( test_bit( ucSegments, SEG_D ) )
      set_bit( PORTA, PA3 );
    else
      clear_bit( PORTA, PA3 );

    if ( test_bit( ucSegments, SEG_E ) )
      set_bit( PORTA, PA4 );
    else
      clear_bit( PORTA, PA4 );

    if ( test_bit( ucSegments, SEG_F ) )
      set_bit( PORTA, PA5 );
    else
      clear_bit( PORTA, PA5 );

    if ( test_bit( ucSegments, SEG_G ) )
      set_bit( PORTA, PA6 );
    else
      clear_bit( PORTA, PA6 );
  }
}
